import { Argument, Option } from 'commander';
import inquirer from 'inquirer';
import { chalk, log } from '../../utils/command-helpers.mjs';
import { getWebSocket } from '../../utils/websockets/index.mjs';
// Source: Source: https://docs.aws.amazon.com/lambda/latest/dg/monitoring-cloudwatchlogs.html#monitoring-cloudwatchlogs-advanced
export const LOG_LEVELS = {
    TRACE: 'TRACE',
    DEBUG: 'DEBUG',
    INFO: 'INFO',
    WARN: 'WARN',
    ERROR: 'ERROR',
    FATAL: 'FATAL',
};
const LOG_LEVELS_LIST = Object.values(LOG_LEVELS).map((level) => level.toLowerCase());
const CLI_LOG_LEVEL_CHOICES_STRING = LOG_LEVELS_LIST.map((level) => ` ${level}`);
function getLog(logData) {
    let logString = '';
    switch (logData.level) {
        case LOG_LEVELS.INFO:
            logString += chalk.blueBright(logData.level);
            break;
        case LOG_LEVELS.WARN:
            logString += chalk.yellowBright(logData.level);
            break;
        case LOG_LEVELS.ERROR:
            logString += chalk.redBright(logData.level);
            break;
        default:
            logString += logData.level;
            break;
    }
    return `${logString} ${logData.message}`;
}
const logsFunction = async (functionName, options, command) => {
    const client = command.netlify.api;
    const { site } = command.netlify;
    const { id: siteId } = site;
    if (options.level && !options.level.every((level) => LOG_LEVELS_LIST.includes(level))) {
        log(`Invalid log level. Choices are:${CLI_LOG_LEVEL_CHOICES_STRING}`);
    }
    const levelsToPrint = options.level || LOG_LEVELS_LIST;
    const { functions = [] } = await client.searchSiteFunctions({ siteId });
    if (functions.length === 0) {
        log(`No functions found for the site`);
        return;
    }
    let selectedFunction;
    if (functionName) {
        selectedFunction = functions.find((fn) => fn.n === functionName);
    }
    else {
        const { result } = await inquirer.prompt({
            name: 'result',
            type: 'list',
            message: 'Select a function',
            choices: functions.map((fn) => fn.n),
        });
        selectedFunction = functions.find((fn) => fn.n === result);
    }
    if (!selectedFunction) {
        log(`Could not find function ${functionName}`);
        return;
    }
    const { a: accountId, oid: functionId } = selectedFunction;
    const ws = getWebSocket('wss://socketeer.services.netlify.com/function/logs');
    ws.on('open', () => {
        ws.send(JSON.stringify({
            function_id: functionId,
            site_id: siteId,
            access_token: client.accessToken,
            account_id: accountId,
        }));
    });
    ws.on('message', (data) => {
        const logData = JSON.parse(data);
        if (!levelsToPrint.includes(logData.level.toLowerCase())) {
            return;
        }
        log(getLog(logData));
    });
    ws.on('close', () => {
        log('Connection closed');
    });
    ws.on('error', (err) => {
        log('Connection error');
        log(err);
    });
};
export const createLogsFunctionCommand = (program) => program
    .command('logs:function')
    .alias('logs:functions')
    .addOption(new Option('-l, --level <levels...>', `Log levels to stream. Choices are:${CLI_LOG_LEVEL_CHOICES_STRING}`))
    .addArgument(new Argument('[functionName]', 'Name of the function to stream logs for'))
    .addExamples([
    'netlify logs:function',
    'netlify logs:function my-function',
    'netlify logs:function my-function -l info warn',
])
    .description('(Beta) Stream netlify function logs to the console')
    .action(logsFunction);
